@@ -2,7 +2,7 @@
  * glade-design-view.c
  *
  * Copyright (C)      2006 Vincent Geddes
- *               2011-2013 Juan Pablo Ugarte
+ *               2011-2016 Juan Pablo Ugarte
  *
  * Authors:
  *   Vincent Geddes <vincent.geddes@gmail.com>
@@ -41,6 +41,7 @@
 #include "glade-design-layout.h"
 #include "glade-design-private.h"
 #include "glade-path.h"
+#include "glade-adaptor-chooser.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -104,39 +105,50 @@ on_layout_size_allocate (GtkWidget *widget, GtkAllocation *alloc, GladeDesignVie
   g_signal_handlers_disconnect_by_func (widget, on_layout_size_allocate, view);
 }
 
+static void
+glade_design_view_update_state (GList *objects, GtkStateFlags state)
+{
+  GList *l;
+
+  for (l = objects; l && l->data; l = g_list_next (l))
+    {
+      GtkWidget *view, *widget = l->data;
+
+      if (GTK_IS_WIDGET (widget) &&
+          gtk_widget_get_visible (widget) &&
+          (view = gtk_widget_get_ancestor (widget, GLADE_TYPE_DESIGN_LAYOUT)))
+        {
+          gtk_widget_set_state_flags (view, state, TRUE);
+        }
+    }
+}
+
 static void
 glade_design_view_selection_changed (GladeProject *project, GladeDesignView *view)
 {
+  GtkWidget *layout;
   GList *selection;
 
+  glade_design_view_update_state (glade_project_toplevels (project),
+                                  GTK_STATE_FLAG_NORMAL);
+
+  if (!(selection = glade_project_selection_get (project)))
+    return;
+
+  glade_design_view_update_state (selection, GTK_STATE_FLAG_SELECTED);
+
   /* Check if its only one widget selected and scroll viewport to show toplevel */
-  if ((selection = glade_project_selection_get (project)) &&
-      g_list_next (selection) == NULL &&
-      GTK_IS_WIDGET (selection->data))
+  if (g_list_next (selection) == NULL &&
+      GTK_IS_WIDGET (selection->data) &&
+      (layout = gtk_widget_get_ancestor (selection->data, GLADE_TYPE_DESIGN_LAYOUT)))
     {
-      GladeWidget *gwidget, *gtoplevel;
-      GObject *toplevel;
-      
-      if (!GLADE_IS_PLACEHOLDER (selection->data) &&
-          (gwidget = glade_widget_get_from_gobject (G_OBJECT (selection->data))) &&
-          (gtoplevel = glade_widget_get_toplevel (gwidget)) &&
-          (toplevel = glade_widget_get_object (gtoplevel)) &&
-          GTK_IS_WIDGET (toplevel))
-        {
-          GtkWidget *layout;
+      GtkAllocation alloc;
+      gtk_widget_get_allocation (layout, &alloc);
 
-          if ((layout = gtk_widget_get_parent (GTK_WIDGET (toplevel))) &&
-              GLADE_IS_DESIGN_LAYOUT (layout))
-            {
-              GtkAllocation alloc;
-              gtk_widget_get_allocation (layout, &alloc);
-
-              if (alloc.x < 0)
-                g_signal_connect (layout, "size-allocate", G_CALLBACK (on_layout_size_allocate), view);
-              else
-                glade_design_layout_scroll (view, alloc.x, alloc.y, alloc.width, alloc.height);
-            }
-        }
+      if (alloc.x < 0)
+        g_signal_connect (layout, "size-allocate", G_CALLBACK (on_layout_size_allocate), view);
+      else
+        glade_design_layout_scroll (view, alloc.x, alloc.y, alloc.width, alloc.height);
     }
 }
 
@@ -301,7 +313,7 @@ glade_design_view_get_property (GObject *object,
 }
 
 static void
-logo_draw (GtkWidget *widget, cairo_t *cr)
+logo_draw (GtkWidget *widget, cairo_t *cr, GdkRGBA *c)
 {
   GtkAllocation alloc;
   gdouble scale;
@@ -310,7 +322,7 @@ logo_draw (GtkWidget *widget, cairo_t *cr)
 
   cairo_save (cr);
   
-  cairo_set_source_rgba (cr, 0, 0, 0, .08);
+  cairo_set_source_rgba (cr, c->red, c->green, c->blue, .06);
 
   scale = MIN ((alloc.width/1.5)/(glade_path_WIDTH), (alloc.height/1.5)/(glade_path_HEIGHT));
 
@@ -324,34 +336,67 @@ logo_draw (GtkWidget *widget, cairo_t *cr)
   cairo_restore (cr);
 }
 
-static gboolean
-glade_design_view_draw (GtkWidget *widget, cairo_t *cr)
+static void
+on_chooser_adaptor_selected (_GladeAdaptorChooser *chooser,
+                             GladeWidgetAdaptor   *adaptor,
+                             GladeProject         *project)
+
 {
-  GladeDesignViewPrivate *priv = GLADE_DESIGN_VIEW (widget)->priv;
-  GdkWindow *window = gtk_widget_get_window (widget);
-  gboolean should_draw = gtk_cairo_should_draw_window (cr, window);
-  gboolean sw_visible = gtk_widget_get_visible (priv->scrolled_window);
+  glade_command_create (adaptor, NULL, NULL, project);
+  gtk_widget_destroy (gtk_widget_get_ancestor (GTK_WIDGET (chooser), GTK_TYPE_POPOVER));
+}
 
-  if (should_draw)
+static gboolean
+glade_design_view_viewport_button_press (GtkWidget       *widget,
+                                         GdkEventButton  *event,
+                                         GladeDesignView *view)
+{
+  GladeDesignViewPrivate *priv = view->priv;
+  GdkRectangle rect = {event->x, event->y, 8, 8};
+  GtkWidget *pop, *chooser;
+  
+  if (event->type != GDK_2BUTTON_PRESS)
+    return FALSE;
+
+  pop = gtk_popover_new (widget);
+  gtk_popover_set_pointing_to (GTK_POPOVER (pop), &rect);
+  gtk_popover_set_position (GTK_POPOVER (pop), GTK_POS_BOTTOM);
+  
+  chooser = _glade_adaptor_chooser_new (GLADE_ADAPTOR_CHOOSER_TOPLEVEL,
+                                        priv->project);
+  g_signal_connect (chooser, "adaptor-selected",
+                    G_CALLBACK (on_chooser_adaptor_selected),
+                    priv->project);
+
+  gtk_container_add (GTK_CONTAINER (pop), chooser);
+  gtk_widget_show (chooser);
+  gtk_widget_show (pop);
+
+  return TRUE;
+}
+
+static gboolean
+glade_design_view_viewport_draw (GtkWidget *widget, cairo_t *cr, GladeDesignView *view)
+{
+  GladeDesignViewPrivate *priv = GLADE_DESIGN_VIEW (view)->priv;
+  GtkStyleContext *context = gtk_widget_get_style_context (widget);
+  GdkRGBA fg_color;
+
+  gtk_style_context_get_color (context, gtk_style_context_get_state (context),
+                               &fg_color);
+
+  logo_draw (widget, cr, &fg_color);
+
+  if (priv->drag_highlight)
     {
-      if (sw_visible)
-        gtk_render_background (gtk_widget_get_style_context (widget),
-                               cr, 0, 0,
-                               gtk_widget_get_allocated_width (widget),
-                               gtk_widget_get_allocated_height (widget));
-      else
-        logo_draw (widget, cr);
-    }
-
-  GTK_WIDGET_CLASS (glade_design_view_parent_class)->draw (widget, cr);
-
-  if (should_draw && sw_visible && priv->drag_highlight)
-    {
-      GtkStyleContext *context = gtk_widget_get_style_context (widget);
       GdkRGBA c;
 
-      gtk_style_context_get_background_color (context, GTK_STATE_FLAG_SELECTED |
+      gtk_style_context_save (context);
+      gtk_style_context_get_background_color (context,
+                                              gtk_style_context_get_state (context) |
+                                              GTK_STATE_FLAG_SELECTED |
                                               GTK_STATE_FLAG_FOCUSED, &c);
+      gtk_style_context_restore (context);
 
       cairo_set_line_width (cr, 2);
       gdk_cairo_set_source_rgba (cr, &c);
@@ -360,10 +405,11 @@ glade_design_view_draw (GtkWidget *widget, cairo_t *cr)
                        gtk_widget_get_allocated_height (widget));
       cairo_stroke (cr);
     }
-  
+
   return FALSE;
 }
 
+
 static void
 glade_design_view_init (GladeDesignView *view)
 {
@@ -386,6 +432,13 @@ glade_design_view_init (GladeDesignView *view)
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   viewport = gtk_viewport_new (NULL, NULL);
+  gtk_widget_add_events (viewport, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+  g_signal_connect (viewport, "button-press-event",
+                    G_CALLBACK (glade_design_view_viewport_button_press),
+                    view);
+  g_signal_connect (viewport, "draw",
+                    G_CALLBACK (glade_design_view_viewport_draw),
+                    view);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
   gtk_container_add (GTK_CONTAINER (viewport), view->priv->layout_box);
   gtk_container_add (GTK_CONTAINER (view->priv->scrolled_window), viewport);
@@ -704,7 +757,6 @@ glade_design_view_class_init (GladeDesignViewClass *klass)
   widget_class->drag_leave = glade_design_view_drag_leave;
   widget_class->drag_data_received = glade_design_view_drag_data_received;
   widget_class->drag_drop = glade_design_view_drag_drop;
-  widget_class->draw = glade_design_view_draw;
   
   g_object_class_install_property (object_class,
                                    PROP_PROJECT,
@@ -716,27 +768,6 @@ glade_design_view_class_init (GladeDesignViewClass *klass)
                                                         G_PARAM_CONSTRUCT_ONLY));
 }
 
-/* Private API */
-void
-_glade_design_view_freeze (GladeDesignView *view)
-{
-  g_return_if_fail (GLADE_IS_DESIGN_VIEW (view));
-  
-  g_signal_handlers_block_by_func (view->priv->project,
-                                   glade_design_view_selection_changed,
-                                   view);
-}
-
-void
-_glade_design_view_thaw   (GladeDesignView *view)
-{
-  g_return_if_fail (GLADE_IS_DESIGN_VIEW (view));
-  
-  g_signal_handlers_unblock_by_func (view->priv->project,
-                                     glade_design_view_selection_changed,
-                                     view);
-}
-
 /* Public API */
 
 GladeProject *
